/*
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
*/

package org.apache.griffin.core.measure.repo;


import org.apache.griffin.core.measure.entity.Measure;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.CrudRepository;

import java.util.List;

public interface MeasureRepo<T extends Measure> extends CrudRepository<T, Long> {

    List<T> findByNameAndDeleted(String name, Boolean deleted);

    List<T> findByDeleted(Boolean deleted);

    List<T> findByOwnerAndDeleted(String owner, Boolean deleted);

    T findByIdAndDeleted(Long id, Boolean deleted);

    @Query("select DISTINCT m.organization from #{#entityName} m where m.deleted = ?1")
    List<String> findOrganizations(Boolean deleted);

    @Query("select m.name from #{#entityName} m " +
            "where m.organization= ?1 and m.deleted= ?2")
    List<String> findNameByOrganization(String organization, Boolean deleted);
}
